=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed August 2008 by Fredo6

# This software is provided as an example of using the Ruby interface to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6Transform.rb
# Original Date	:  07 Jan 2009 - version 3.0
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  A utility library about Transformation for LibFredo6-compliant scripts.
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module G6

#--------------------------------------------------------------------------------------------------------------
# Tapering Transformation
#--------------------------------------------------------------------------------------------------------------			 				   

class TR_Tapering

#Create a new Tapering transformation
def initialize(origin, axes, xscale, yscale, height)
	@origin = origin
	@axes = axes
	@xscale = xscale
	@yscale = yscale
	@height = height
	@tr_axe = Geom::Transformation.axes origin, axes[0], axes[1], axes[2]
	@tr_axeinv = @tr_axe.inverse
	@tr_identity = Geom::Transformation.new
end

#Apply the transformation to a point
def *(entity)
	if entity.class == Geom::Point3d
		return transform_point(entity)
	end
	nil
end

def transform_point(pt)
	pt = @tr_axeinv * pt
	zscale = pt.z / @height
	pt.x *= (1.0 + (@xscale - 1.0) * zscale)
	pt.y *= (1.0 + (@yscale - 1.0) * zscale)
	return @tr_axe * pt
end

def transform_entities(entities)
	recurse nil, entities, @tr_identity
end

def prepare(entities)
	lcomp = []
	lgroup = []
	entities.each do |e|
		if e.class == Sketchup::Edge && e.curve
			e.explode_curve
		elsif e.class == Sketchup::ComponentInstance	
			lcomp.push e
			prepare e.definition.entities
		elsif e.class == Sketchup::Group	
			lgroup.push e
			prepare e.entities
		end
	end
	lcomp.reverse.each { |e| e.make_unique }
	lgroup.reverse.each { |e| e.make_unique }
end

def recurse(model, entities, t)
	hvertex = {}
	model = entities[0].parent unless model
	entities.each do |e|
		if e.class == Sketchup::Face
			e.edges.each { |edge| edge.explode_curve if edge.curve }
			e.vertices.each { |v| hvertex[v.to_s] = v }
		elsif e.class == Sketchup::Edge
			e.explode_curve if e.curve
			hvertex[e.start.to_s] = e.start
			hvertex[e.end.to_s] = e.end
		elsif e.class == Sketchup::ComponentInstance
			e = e.make_unique unless e.definition.instances == 1
			recurse e.definition, e.definition.entities, t * e.transformation
		elsif e.class == Sketchup::Group	
			e = e.make_unique unless SU_MAJOR_VERSION >= 7
			recurse nil, e.entities, t * e.transformation
		end
	end

	vectors = []
	lvertices = []
	tinv = t.inverse
	hvertex.each do |key, v|
		pt = t * v.position
		ptgoal = transform_point pt
		#ptgoal = self * pt
		ptgoal2 = tinv * ptgoal
		#vectors.push (t.inverse) * (pt.vector_to(ptgoal))
		vectors.push v.position.vector_to(ptgoal2)
		lvertices.push v
	end
	
	model.entities.transform_by_vectors lvertices, vectors
end
	
end	#class TR_Tapering

end #Module Traductor

